﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using RTedge;   // RT-edge system API

namespace EgDDEsvr
{
    /// <summary>
    /// 監視タグ情報
    /// </summary>
    public class AdviseTagInfo
    {
        /// <summary>
        /// タグ名
        /// </summary>
        public string TagName
        {
            get { return m_srTag; }
            set { m_srTag = value; }
        }
        private string m_srTag;

        /// <summary>
        /// タグタイプ(データ型)
        /// </summary>
        public ushort TagType
        {
            get { return m_usTagType; }
            set { m_usTagType = value; }
        }
        private ushort m_usTagType;

        /// <summary>
        /// タグ値
        /// </summary>
        public TagVar Data
        {
            get { return m_unData; }
            set { m_unData = value; }
        }
        private TagVar m_unData;

        /// <summary>
        /// タグ値(文字列変換後)
        /// </summary>
        public string StrData
        {
            get { return m_srData; }
            set { m_srData = value; }
        }
        private string m_srData;

        /// <summary>
        /// 強制送信フラグ
        /// </summary>
        public bool Force
        {
            get { return m_boForce; }
            set { m_boForce = value; }
        }
        private bool m_boForce;

        /// <summary>
        /// 監視要求元クライアントのハンドル値
        /// NOTE: 同一タグを複数クライアントが参照するケースを想定してクライアント識別情報を保持します
        /// </summary>
        public List<IntPtr> Client
        {
            get { return m_pCli; }
            set { m_pCli = value; }
        }
        private List<IntPtr> m_pCli;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="srTagName">タグ名</param>
        /// <param name="usTagType">データタイプ</param>
        /// <param name="val">タグ値</param>
        public AdviseTagInfo(string srTagName, ushort usTagType, object val, IntPtr pCli)
        {
            m_boForce = true;       // 初回は、Advise送信必要のため強制送信フラグをON
            m_srTag = srTagName;
            m_usTagType = usTagType;
            // タグのデータ型毎に処理
            switch (m_usTagType)
            {
                case (ushort)EGDEFINE.egTagDataType.Boolean: m_unData.boVal = (Boolean)val; m_srData = m_unData.boVal.ToString(); break;
                case (ushort)EGDEFINE.egTagDataType.SByte: m_unData.sbVal = (SByte)val; m_srData = m_unData.sbVal.ToString(); break;
                case (ushort)EGDEFINE.egTagDataType.Byte: m_unData.byVal = (Byte)val; m_srData = m_unData.byVal.ToString(); break;
                case (ushort)EGDEFINE.egTagDataType.Int16: m_unData.shVal = (Int16)val; m_srData = m_unData.shVal.ToString(); break;
                case (ushort)EGDEFINE.egTagDataType.UInt16: m_unData.usVal = (UInt16)val; m_srData = m_unData.usVal.ToString(); break;
                case (ushort)EGDEFINE.egTagDataType.Int32: m_unData.inVal = (Int32)val; m_srData = m_unData.inVal.ToString(); break;
                case (ushort)EGDEFINE.egTagDataType.UInt32: m_unData.uiVal = (UInt32)val; m_srData = m_unData.uiVal.ToString(); break;
                case (ushort)EGDEFINE.egTagDataType.Int64: m_unData.loVal = (Int64)val; m_srData = m_unData.loVal.ToString(); break;
                case (ushort)EGDEFINE.egTagDataType.UInt64: m_unData.ulVal = (UInt64)val; m_srData = m_unData.ulVal.ToString(); break;
                case (ushort)EGDEFINE.egTagDataType.Float: m_unData.flVal = (float)val; m_srData = m_unData.flVal.ToString(); break;
                case (ushort)EGDEFINE.egTagDataType.Double: m_unData.doVal = (double)val; m_srData = m_unData.doVal.ToString(); break;
                default: break;
            }
            Client = new List<IntPtr>();
            Client.Add(pCli);
        }

        /// <summary>
        /// クライアント情報の追加
        /// </summary>
        /// <param name="hCli"></param>
        public void AddClient(IntPtr hCli)
        {
            if (Client == null)
            {
                Client = new List<IntPtr>();
            }

            if (hCli != null)
            {
                Client.Add(hCli);
            }
        }

        /// <summary>
        /// クライアント情報の削除
        /// </summary>
        /// <param name="hCli"></param>
        public void DelClient(IntPtr hCli)
        {
            if (hCli != null)
            {
                Client.Remove(hCli);
            }
        }

        /// <summary>
        /// タグ値更新処理
        /// NOTE: 前回値からの変化有無を返却
        /// </summary>
        /// <param name="val"></param>
        /// <returns>True:変化あり、False:変化なし</returns>
        public bool UpdateData(object val)
        {
            bool ret = false;

            // タグのデータ型毎に処理
            // NOTE:値変化時のみタグ値を文字列変換します
            switch (m_usTagType)
            {
                case (ushort)EGDEFINE.egTagDataType.Boolean: if (m_unData.boVal != (Boolean)val) { m_unData.boVal = (Boolean)val; m_srData = m_unData.boVal.ToString(); ret = true; } break;
                case (ushort)EGDEFINE.egTagDataType.SByte: if (m_unData.sbVal != (SByte)val) { m_unData.sbVal = (SByte)val; m_srData = m_unData.sbVal.ToString(); ret = true; } break;
                case (ushort)EGDEFINE.egTagDataType.Byte: if (m_unData.byVal != (Byte)val) { m_unData.byVal = (Byte)val; m_srData = m_unData.byVal.ToString(); ret = true; } break;
                case (ushort)EGDEFINE.egTagDataType.Int16: if (m_unData.shVal != (Int16)val) { m_unData.shVal = (Int16)val; m_srData = m_unData.shVal.ToString(); ret = true; } break;
                case (ushort)EGDEFINE.egTagDataType.UInt16: if (m_unData.usVal != (UInt16)val) { m_unData.usVal = (UInt16)val; m_srData = m_unData.usVal.ToString(); ret = true; } break;
                case (ushort)EGDEFINE.egTagDataType.Int32: if (m_unData.inVal != (Int32)val) { m_unData.inVal = (Int32)val; m_srData = m_unData.inVal.ToString(); ret = true; } break;
                case (ushort)EGDEFINE.egTagDataType.UInt32: if (m_unData.uiVal != (UInt32)val) { m_unData.uiVal = (UInt32)val; m_srData = m_unData.uiVal.ToString(); ret = true; } break;
                case (ushort)EGDEFINE.egTagDataType.Int64: if (m_unData.loVal != (Int64)val) { m_unData.loVal = (Int64)val; m_srData = m_unData.loVal.ToString(); ret = true; } break;
                case (ushort)EGDEFINE.egTagDataType.UInt64: if (m_unData.ulVal != (UInt64)val) { m_unData.ulVal = (UInt64)val; m_srData = m_unData.ulVal.ToString(); ret = true; } break;
                case (ushort)EGDEFINE.egTagDataType.Float: if (m_unData.flVal != (float)val) { m_unData.flVal = (float)val; m_srData = m_unData.flVal.ToString(); ret = true; } break;
                case (ushort)EGDEFINE.egTagDataType.Double: if (m_unData.doVal != (double)val) { m_unData.doVal = (double)val; m_srData = m_unData.doVal.ToString(); ret = true; } break;
                default: break;
            }

            if (m_boForce)          // 強制送信フラグONの場合
            {
                ret = true;         // 変化ありとしてAdvise()コールさせるためtrueをセット
                m_boForce = false;  // 強制送信フラグOFF
            }
            return ret;     // 値変化有無を返却
        }
    }
}
