﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Windows.Forms;
using System.Diagnostics;
using RTedge;   // RT-edge system API

namespace EgDDEsvr
{
    public partial class MainForm : Form
    {
        public string ServName;                 // サービス名 (実行ファイル名から取得)
        public edgeCode EgProc = null;          // RTedge フレームワーク+APIを使用したコードクラス
        public EgDDEServer m_DDESvr;            // DDE通信処理クラス

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            // 実行ファイル名取得
            ServName = System.IO.Path.GetFileNameWithoutExtension(System.Reflection.Assembly.GetEntryAssembly().Location);

            // フォームの非表示
            ShowInTaskbar = false;                      // タスクバーに表示しない
            WindowState = FormWindowState.Minimized;    // 最小化で起動(タスクトレイに入る)

            //// タスクトレイに表示
            //notifyIcon1.Visible = true;
            //notifyIcon1.Text = ServName;
        }

        /// <summary>
        /// フォームロード
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            int inRet = EDGECONST.ERR_EGDDESVR_OK;

            EgProc = new edgeCode(ServName);    // RTedge処理クラスの作成
            EgProc.mInstMainClose = Stop;       // フレームワーク内終了メッセージ処理時の関数指定
            inRet = EgProc.InitService();       // RTedge処理クラスの初期化
            if (inRet != EDGECONST.ERR_EGDDESVR_OK)
            {
                // Edgeフレームワーク初期化失敗
                MessageBox.Show(string.Format("Failed: initial {0}. {1} (0x{2})", ServName, inRet.ToString(), inRet.ToString("X8")), ServName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                // アプリケーションの終了
                Environment.Exit(0);
                return;
            }

            m_DDESvr = new EgDDEServer(EgProc.MyTags.AppName, EgProc);   // DDE通信クラス生成
            bool bRet = m_DDESvr.Start();       // DDE Server 開始
            if (bRet == false)
            {
                // サーバー開始失敗
                inRet = EDGECONST.ERR_EGDDESVR_INIT;
                EgProc.MyTags.SetLastError(EDGECONST.ERR_EGDDESVR_INIT);    // サービスプロパティタグ(発生した最新のエラーコード)を更新
                MessageBox.Show(string.Format("Failed: start DDE Server. {0} (0x{1})" , inRet.ToString(), inRet.ToString("X8")), ServName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        public void Stop()
        {
            try
            {
                m_DDESvr.End();                 // DDE通信クラス終了処理
                EgProc.KillService();           // サービス削除関数呼び出し
                Environment.Exit(0);            // アプリケーションの終了
            }
            catch (Exception ex)
            {
                EgProc.LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// 終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                // サービス停止MSG送信
                EgProc.SendMyMbox((int)EGDEFINE.EGMSG.EM_SERVICE_STOP);

                // メッセージハンドラでアプリケーションを終了させるためフォームを閉じない
                e.Cancel = true;
            }
            catch (Exception ep)
            {
                Debug.WriteLine(ep.Message);
            }
        }


        /// <summary>
        /// タスクトレイアイコンでの終了メニュークリック処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void 終了ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // フォームを閉じる
            Close();
        }
    }
}
