﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using RTedge;   // RT-edge system API

namespace EgDDEsvr
{
    ////
    /// サービスプロパティタグ管理
    //
    public class MYTAG
    {
        private edge_API _egAPI;
        private EDGECONST _egConst;

        public byte State;                  // インジケータ.エッジシステム接続状態
        public bool Run;                    // インジケータ.サービス実行状態
        public bool Error;                  // インジケータ.サービス異常状態
        public UInt32 Live;                 // インジケータ.サービス実行カウンタ

        public UInt32 Cycle;                // プロパティ.Adviseタグ監視周期(ms)
        public string AppName;              // プロパティ.アプリケーション名 ※DDE接続設定
        public string TopicName;            // プロパティ.トピック名 ※DDE接続設定

        public UInt32 DdeClientNum;         // 現在のDDEクライアント接続数
        public UInt32 DdeCliOpen;           // DDEクライアントからの接続開始回数
        public UInt32 DdeCliClose;          // DDEクライアントからの接続終了回数
        public UInt32 DdeWriteReqCount;     // DDEクライアントからの書き込み要求回数
        public UInt32 DdeReadReqCount;      // DDEクライアントからの読み込み要求回数
        public UInt32 DdeMoniterReqCount;   // DDEクライアントからの監視要求回数
        public UInt32 DdeMoniterNum;        // 現在の監視アイテム数
        public Int32 DdeLastErr;            // DDE通信スレッド内で発生した最新のエラーコード
        public UInt32 ReadLive;             // タグ監視処理の稼働カウンタ
        public bool ReadDelay;              // タグ監視処理の遅延状態
        public UInt32 ReadDelayCount;       // タグ監視処理遅延カウンタ
        public UInt32 AdviseCount;          // Adviseをコールした回数(タグ値が変化した回数)
        public float ReadCycleTime;         // 平均処理時間(ms単位)
        public float ReadCycleMin;          // 最短処理時間(ms単位)
        public float ReadCycleMax;          // 最長処理時間(ms単位)
        public UInt32 MsgRUN;               // EM_SERVICE_RUNを受信した回数
        public UInt32 MsgSTOP;              // EM_SERVICE_STOPを受信した回数
        public UInt32 MsgUnknown;           // 未サポートのメッセージを受信した回数

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="egProc"></param>
        /// <param name="egConst"></param>
        public MYTAG(edge_API egProc, EDGECONST egConst)
        {
            _egAPI = egProc;
            _egConst = egConst;

            State = 0;                  // インジケータ.エッジシステム接続状態
            Run = false;                // インジケータ.サービス実行状態
            Error = false;              // インジケータ.サービス異常状態
            Live = 0;                   // インジケータ.サービス実行カウンタ

            Cycle = 1000;               // プロパティ.Adviseタグ監視周期(ms)
            AppName = "";               // プロパティ.アプリケーション名 ※DDE接続設定
            TopicName = "";             // プロパティ.トピック名 ※DDE接続設定

            DdeClientNum = 0;           // 現在のDDEクライアント接続数
            DdeCliOpen = 0;             // DDEクライアントからの接続開始回数
            DdeCliClose = 0;            // DDEクライアントからの接続終了回数
            DdeWriteReqCount = 0;       // DDEクライアントからの書き込み要求回数
            DdeReadReqCount = 0;        // DDEクライアントからの読み込み要求回数
            DdeMoniterReqCount = 0;     // DDEクライアントからの監視要求回数
            DdeMoniterNum = 0;          // 現在の監視アイテム数
            DdeLastErr = 0;             // DDE通信スレッド内で発生した最新のエラーコード
            ReadLive = 0;               // タグ監視処理の稼働カウンタ
            ReadDelay = false;          // タグ監視処理の遅延状態
            ReadDelayCount = 0;         // タグ監視処理遅延カウンタ
            AdviseCount = 0;            // Adviseをコールした回数(タグ値が変化した回数)
            ReadCycleTime = 0;          // 平均処理時間(ms単位)
            ReadCycleMin = 0;           // 最短処理時間(ms単位)
            ReadCycleMax = 0;           // 最長処理時間(ms単位)
            MsgRUN = 0;                 // EM_SERVICE_RUNを受信した回数
            MsgSTOP = 0;                // EM_SERVICE_STOPを受信した回数
            MsgUnknown = 0;             // 未サポートのメッセージを受信した回数
        }


        /// <summary>
        /// 最新のエラーコード更新
        /// </summary>
        /// <param name="errCode"></param>
        public void SetLastError(int errCode)
        {
            if (DdeLastErr != errCode)
            {
                DdeLastErr = errCode;    // サービスプロパティタグ(最新のエラーコード)を更新
                _egAPI.EgTagWrite(_egConst.TAG_MYSERV_DDE_LastErr, DdeLastErr);
            }
        }

        /// <summary>
        /// タグ監視処理時間更新
        /// NOTE: 関連するサービスタグを一括更新
        /// </summary>
        /// <param name="time"></param>
        public void Update_OnAdviseTime(float time)
        {
            // ---------------------------------
            // 今回の処理時間更新
            // ---------------------------------
            ReadCycleTime = time;
            _egAPI.EgTagWrite(_egConst.TAG_MYSERV_READ_CycleTime, ReadCycleTime);

            // ---------------------------------
            // 最短処理時間更新
            // ---------------------------------
            if ((ReadCycleMin == 0) 
                || ((ReadCycleMin > time) && (time != 0)))
            {
                ReadCycleMin = time;
                _egAPI.EgTagWrite(_egConst.TAG_MYSERV_READ_CycleMin, ReadCycleMin);
            }
            // ---------------------------------
            // 最長処理時間更新
            // ---------------------------------
            if (ReadCycleMax < time)
            {
                ReadCycleMax = time;
                _egAPI.EgTagWrite(_egConst.TAG_MYSERV_READ_CycleMax, ReadCycleMax);
            }
            // ---------------------------------
            // 処理遅延判定
            // ---------------------------------
            if (Cycle < time)           // 遅延あり
            {
                ReadDelay = true;       // 遅延状態更新
                _egAPI.EgTagWrite(_egConst.TAG_MYSERV_READ_Delay, true);
                ReadDelayCount++;       // 遅延カウント更新
                _egAPI.EgTagWrite(_egConst.TAG_MYSERV_READ_DelayCount, ReadDelayCount);
            }
            else if (Cycle > time && ReadDelay == true) // 遅延なし、かつ前回遅延状態
            {
                ReadDelay = false;      // 遅延状態更新
                _egAPI.EgTagWrite(_egConst.TAG_MYSERV_READ_Delay, false);
            }
            else { }    // 上記以外は、タグ更新不要
        }
    }

}
