﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using RTedge;   // RT-edge system API
using System.Runtime.CompilerServices;
using System.Timers;

namespace EgDDEsvr
{
    /// <summary>
    /// RT-edge API/フレームワーク 各種値定義クラス
    /// </summary>
    public class EDGECONST
    {
        /// <summary>
        /// 各種定義追加
        /// </summary>
        public const string TAG_PREFIX = "SERVICE.";            // サービスプロパティタグ名先頭に付与する文字列
        public const byte SERVICE_UP = 1;                       // 起動中
        public const byte SERVICE_DOWN = 0;                     // 停止中
        public const string DEFAULT_TOPICNAME = "TagValue";     // トピック名のデフォルト値
        public const int DEFAULT_CYCLE = 1000;                  // Adviseタグ監視周期のデフォルト値(ms)
        public const int MIN_CYCLE = 100;                       // Adviseタグ監視周期の最小値(ms)

        ////
        /// エラーコード
        //
        public const int ERR_EGDDESVR_OK                = 0x00000000;   // 正常
        public const int ERR_EGDDESVR_INIT              = 0x01000001;   // DDEサーバーの初期化に失敗しました
        public const int ERR_EGDDESVR_CYCLE             = 0x01000002;   // タグ監視処理周期の設定が未指定、または範囲外です
        public const int ERR_EGDDESVR_FORMAT            = 0x02000001;   // 未対応のDDE通信フォーマットです。
        public const int ERR_EGDDESVR_TOPICNAME         = 0x02000002;   // DDEトピック名が不正です。
        public const int ERR_EGDDESVR_ITEMNAME          = 0x02000003;   // DDEアイテム名が指定されていません。
        public const int ERR_EGDDESVR_TAG_NOEXIST       = 0x02000004;   // 指定されたDDEアイテム名に該当するタグは存在しません。
        public const int ERR_EGDDESVR_CLIENT_NOINFO     = 0x02000006;   // クライアント接続情報がありません

        ////
        /// サービス名
        //
        public string SERVICENAME;                     // サービス名 ※実行プロセス名から取得
        ////
        /// サービスインジケータ名
        //        
        public string IND_MYSERV_STATUS;               // サービス起動
        public string IND_MYSERV_ERROR;                // サービスエラー発生中
        public string IND_MYSERV_RUN;                  // サービス実行中
        public string IND_MYSERV_LIVE;                 // サービス実行カウンタ

        ////
        /// サービスプロパティタグ名
        //
        public string PROP_MYSERV_APPNAME;             // DDE接続設定:アプリケーション名(サーバー名)
        public string PROP_MYSERV_TOPICNAME;           // DDE接続設定:トピック名
        public string PROP_MYSERV_CYCLE;               // Adviseタグ監視周期(ms)

        ////
        /// サービス提供タグ名
        //
        public  string TAG_MYSERV_DDE_CliNum;           // 現在のDDEクライアント接続数
        public  string TAG_MYSERV_DDE_ConnectedOpen;    // DDEクライアントからの接続開始回数
        public  string TAG_MYSERV_DDE_ConnectedClose;   // DDEクライアントからの接続終了回数
        public  string TAG_MYSERV_DDE_WriteReqCount;    // DDEクライアントからの書き込み要求回数
        public  string TAG_MYSERV_DDE_ReadReqCount;     // DDEクライアントからの読み込み要求回数
        public  string TAG_MYSERV_DDE_MoniterReqCount;  // DDEクライアントからの監視要求回数
        public  string TAG_MYSERV_DDE_MoniterNum;       // 現在の監視アイテム数
        public  string TAG_MYSERV_DDE_LastErr;          // DDE通信スレッド内で発生した最新のエラーコード
        public  string TAG_MYSERV_READ_Live;            // タグ監視処理の稼働カウンタ
        public  string TAG_MYSERV_READ_Delay;           // タグ監視処理の遅延状態
        public  string TAG_MYSERV_READ_DelayCount;      // タグ監視処理遅延カウンタ
        public  string TAG_MYSERV_READ_CycleTime;       // 平均処理時間(ms単位)
        public  string TAG_MYSERV_READ_CycleMin;        // 最短処理時間(ms単位)
        public  string TAG_MYSERV_READ_CycleMax;        // 最長処理時間(ms単位)
        public  string TAG_MYSERV_READ_AdviseCount;     // Adviseをコールした回数(タグ値が変化した回数)
        public  string TAG_MYSERV_MSG_RUN;              // EM_SERVICE_RUNを受信した回数
        public  string TAG_MYSERV_MSG_STOP;             // EM_SERVICE_STOPを受信した回数
        public  string TAG_MYSERV_MSG_Unknown;          // 未サポートのメッセージを受信した回数

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="Name"></param>
        public EDGECONST(string Name)
        {
            // NOTE: サービス名から各タグ名称を生成します。
            SET_EDGECONST(Name);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="Name"></param>
        public void  SET_EDGECONST(string Name)
        {
            // NOTE: サービス名から各タグ名称を生成します。
            SERVICENAME = Name;
            IND_MYSERV_STATUS = TAG_PREFIX + SERVICENAME + ".Status";             // サービス起動
            IND_MYSERV_ERROR = TAG_PREFIX + SERVICENAME + ".Error";              // サービスエラー発生中
            IND_MYSERV_RUN = TAG_PREFIX + SERVICENAME + ".Run";                // サービス実行中
            IND_MYSERV_LIVE = TAG_PREFIX + SERVICENAME + ".Live";               // サービス実行カウンタ

            PROP_MYSERV_CYCLE = TAG_PREFIX + SERVICENAME + ".Cycle";              // Adviseタグ監視周期(ms)
            PROP_MYSERV_APPNAME = TAG_PREFIX + SERVICENAME + ".AppName";            // アプリケーション名
            PROP_MYSERV_TOPICNAME = TAG_PREFIX + SERVICENAME + ".TopicName";          // トピック名

            TAG_MYSERV_DDE_CliNum = TAG_PREFIX + SERVICENAME + ".DDE.ClientNum";      // 現在のDDEクライアント接続数
            TAG_MYSERV_DDE_ConnectedOpen = TAG_PREFIX + SERVICENAME + ".DDE.ConnectedOpen";  // DDEクライアントからの接続開始回数
            TAG_MYSERV_DDE_ConnectedClose = TAG_PREFIX + SERVICENAME + ".DDE.ConnectedClose"; // DDEクライアントからの接続終了回数
            TAG_MYSERV_DDE_WriteReqCount = TAG_PREFIX + SERVICENAME + ".DDE.WriteReqCount";  // DDEクライアントからの書き込み要求回数
            TAG_MYSERV_DDE_ReadReqCount = TAG_PREFIX + SERVICENAME + ".DDE.ReadReqCount";   // DDEクライアントからの読み込み要求回数
            TAG_MYSERV_DDE_MoniterReqCount = TAG_PREFIX + SERVICENAME + ".DDE.MoniterReqCount";// DDEクライアントからの監視要求回数
            TAG_MYSERV_DDE_MoniterNum = TAG_PREFIX + SERVICENAME + ".DDE.MoniterNum";     // 現在の監視アイテム数
            TAG_MYSERV_DDE_LastErr = TAG_PREFIX + SERVICENAME + ".DDE.LastErr";        // DDE通信スレッド内で発生した最新のエラーコード
            TAG_MYSERV_READ_Live = TAG_PREFIX + SERVICENAME + ".READ.Live";          // タグ監視処理の稼働カウンタ
            TAG_MYSERV_READ_Delay = TAG_PREFIX + SERVICENAME + ".READ.Delay";         // タグ監視処理の遅延状態
            TAG_MYSERV_READ_DelayCount = TAG_PREFIX + SERVICENAME + ".READ.DelayCount";    // タグ監視処理遅延カウンタ
            TAG_MYSERV_READ_CycleTime = TAG_PREFIX + SERVICENAME + ".READ.CycleTime";     // 平均処理時間(ms単位)
            TAG_MYSERV_READ_CycleMin = TAG_PREFIX + SERVICENAME + ".READ.CycleMin";      // 最短処理時間(ms単位)
            TAG_MYSERV_READ_CycleMax = TAG_PREFIX + SERVICENAME + ".READ.CycleMax";      // 最長処理時間(ms単位)
            TAG_MYSERV_READ_AdviseCount = TAG_PREFIX + SERVICENAME + ".READ.AdviseCount";   // Adviseをコールした回数(タグ値が変化した回数)
            TAG_MYSERV_MSG_RUN = TAG_PREFIX + SERVICENAME + ".MSG.RUN";            // EM_SERVICE_RUNを受信した回数
            TAG_MYSERV_MSG_STOP = TAG_PREFIX + SERVICENAME + ".MSG.STOP";           // EM_SERVICE_STOPを受信した回数
            TAG_MYSERV_MSG_Unknown = TAG_PREFIX + SERVICENAME + ".MSG.Unknown";        // 未サポートのメッセージを受信した回数

        }
    }
    /// <summary>
    /// RT-edge API/フレームワーク実装クラス
    /// </summary>
    public class edgeCode
    {
        ////
        /// edge_APIクラス インスタンス化 
        //
        public edge_API EGAPI = null;                   // edge APIのインスタンス
        public MYTAG MyTags = null;                     // 自身のサービスタグ
        public EDGECONST EGCONST = null;                // 各種定義値
        Timer ServiceIndLiveTimer;                      // Liveカウンタ更新用タイマ

        ////
        /// メイン処理終了用コールバック関数
        //
        public delegate void CallbackMainClose();
        public CallbackMainClose mInstMainClose = null;
        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="myMsgHndlers">メッセージハンドラー関数を指定</param>
        public edgeCode(string Name) 
        {
            EGAPI = new edge_API();
            EGCONST = new EDGECONST(Name);
            MyTags = new MYTAG(EGAPI, EGCONST);
        }
        
        /// <summary>
        /// Edgeメッセージハンドラ
        /// </summary>
        /// <param name="senderName">メッセージ送信元</param>
        /// <param name="messNo">メッセージ番号</param>
        /// <param name="paramArray">メッセージパラメータ</param>
        /// <returns></returns>
        object WMLOCK = new object();
        public Int32 MsgHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                switch (messNo) //メッセージ番号毎に処理
                {
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_RUN:    //サービス開始指令
                        // サービスプロパティタグ(EM_SERVICE_RUNを受信した回数)を更新
                        MyTags.MsgRUN++;
                        EGAPI.EgTagWrite(EGCONST.TAG_MYSERV_MSG_RUN, MyTags.MsgRUN);

                        StartService();
                        break;
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:   // サービス終了指令
                        // サービスプロパティタグ(EM_SERVICE_STOPを受信した回数)を更新
                        MyTags.MsgSTOP++;
                        EGAPI.EgTagWrite(EGCONST.TAG_MYSERV_MSG_STOP, MyTags.MsgSTOP);

                        if (mInstMainClose != null)
                            mInstMainClose();                   // コールバック関数からは戻りません
                        else
                            KillService();
                        break;
                    default:
                        //// サービスプロパティタグ(未サポートのメッセージを受信した回数)を更新
                        //MyTags.MsgUnknown++;
                        //EGAPI.EgTagWrite(EGCONST.TAG_MYSERV_MSG_Unknown, MyTags.MsgUnknown);
                        break;
                }
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        /// <summary>
        /// Liveインジケータの更新（タイマイベント）
        /// </summary>
        /// <param name="source"></param>
        /// <param name="e"></param>
        private void OnTimedLiveIndicaterEvent(Object source, System.Timers.ElapsedEventArgs e)
        {
            try
            {
                // インジケータ更新
                ((Timer)source).Stop();
                MyTags.Live++;
                EGAPI.EgTagWrite(EGCONST.IND_MYSERV_LIVE, MyTags.Live);
            }
            finally
            {
                // 次のタイマーイベントを許可
                ((Timer)source).Start();
            }
        }

        /// <summary>
        /// サービスタグ生成 ※すでに生成されている場合は無視して継続します
        /// </summary>
        /// <returns></returns>
        public int CreateServiceTags()
        {
            int result = 0;
            try
            {
                // サービスタグ類の生成
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_DDE_CliNum, (ushort)EGDEFINE.egTagDataType.Byte, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_DDE_ConnectedOpen, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_DDE_ConnectedClose, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_DDE_WriteReqCount, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_DDE_ReadReqCount, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_DDE_MoniterReqCount, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_DDE_MoniterNum, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_DDE_LastErr, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_READ_Live, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_READ_Delay, (ushort)EGDEFINE.egTagDataType.Boolean, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_READ_DelayCount, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_READ_CycleTime, (ushort)EGDEFINE.egTagDataType.Float, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_READ_CycleMin, (ushort)EGDEFINE.egTagDataType.Float, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_READ_CycleMax, (ushort)EGDEFINE.egTagDataType.Float, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_READ_AdviseCount, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_MSG_RUN, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_MSG_STOP, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_MSG_Unknown, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
            }
            catch (Exception ex)
            {
                edge_FW._C_EgFW_PrintLog(EGAPI.EGFW.ServiceRealName, edge_FW.EG_LOG_WRN, "Failed: CreateServiceTags"  + " 0x" + ex.HResult.ToString("X8"));
            }
            return result;
        }
        /// <summary>
        /// サービスインジケータの更新 ※更新に失敗しても無視して継続します
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        public int UpdateIndicator()
        {
            try
            {
                // インジケータタグはフレームワークによって自動生成済
                int result = 0;
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_STATUS, MyTags.State);   // サービス起動状態
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_RUN, MyTags.Run);        // サービス実行状態
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_LIVE, MyTags.Live);      // サービス実行カウンタ
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_ERROR, MyTags.Error);    // サービス異常状態
                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービスプロパティの取得 ※取得できない時はデフォルト設定値を採用しタグを生成
        /// </summary>
        /// <returns></returns>
        public int GetProperty()
        {
            try
            {
                // プロパティタグはXML定義可能　無いときはデフォルト設定値を採用しタグを生成します
                int result = 0;
                object value = new object();

                // Adviseタグ監視周期の取得
                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_CYCLE, ref value);
                if (result == 0)
                {
                    if ((uint)value < EDGECONST.MIN_CYCLE)  // 最小値未満の場合
                    {
                        MyTags.Cycle = EDGECONST.DEFAULT_CYCLE;             // 最小値未満の場合はデフォルト値を使用
                        EGAPI.EgTagWrite(EGCONST.PROP_MYSERV_CYCLE, MyTags.Cycle);  // タグ更新
                        edge_FW._C_EgFW_PrintLog(EGAPI.EGFW.ServiceRealName, edge_FW.EG_LOG_WRN, string.Format("{0} out of range value. Tag was generated by default. ", EGCONST.PROP_MYSERV_CYCLE));
                        MyTags.SetLastError(EDGECONST.ERR_EGDDESVR_CYCLE);  // サービスプロパティタグ(最新のエラーコード)を更新
                    }
                    else
                    {
                        MyTags.Cycle = (uint)value;                         // 取得値を使用
                    }
                }
                else
                {
                    // タグが生成されていない場合はデフォルト値で生成する
                    MyTags.Cycle = EDGECONST.DEFAULT_CYCLE;                 // デフォルト値でタグを生成する
                    result = EGAPI.EgTagCreateEx(EGCONST.PROP_MYSERV_CYCLE, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");
                    result = EGAPI.EgTagWrite(EGCONST.PROP_MYSERV_CYCLE, MyTags.Cycle);
                    edge_FW._C_EgFW_PrintLog(EGAPI.EGFW.ServiceRealName, edge_FW.EG_LOG_WRN, string.Format("{0} tag not exist. Tag was generated by default. ", EGCONST.PROP_MYSERV_CYCLE));
                    MyTags.SetLastError(EDGECONST.ERR_EGDDESVR_CYCLE);      // サービスプロパティタグ(最新のエラーコード)を更新
                }
                // アプリケーション名の取得
                result = EGAPI.EgTagGetProperty(EGCONST.PROP_MYSERV_APPNAME, "Address", ref value);
                if (result == 0)
                {
                    if (string.IsNullOrEmpty((string)value))    // 取得したAddressの判定
                    {
                        MyTags.AppName = EGCONST.SERVICENAME;               // Addressが未定義の場合は、デフォルト値(※サービス名)を使用
                        edge_FW._C_EgFW_PrintLog(EGAPI.EGFW.ServiceName, edge_FW.EG_LOG_WRN, string.Format("Application name is not set. default:[{0}]", MyTags.AppName));
                    }
                    else
                    {
                        MyTags.AppName = (string)value;                     // 取得値を使用
                    }
                }
                else
                {
                    // 生成されていない場合はデフォルト値でタグ生成する
                    MyTags.AppName = EGCONST.SERVICENAME;                   // デフォルト値(※サービス名)
                    result = EGAPI.EgTagCreateEx(EGCONST.PROP_MYSERV_APPNAME, (ushort)EGDEFINE.egTagDataType.Boolean, MyTags.AppName, "");
                    edge_FW._C_EgFW_PrintLog(EGAPI.EGFW.ServiceName, edge_FW.EG_LOG_WRN, string.Format("{0} tag not exist. Tag was generated by default. ", EGCONST.PROP_MYSERV_APPNAME));
                }

                // トピック名の取得
                result = EGAPI.EgTagGetProperty(EGCONST.PROP_MYSERV_TOPICNAME, "Address", ref value);
                if (result == 0)
                {
                    if (string.IsNullOrEmpty((string)value))    // 取得したAddressの判定
                    {
                        MyTags.TopicName = EDGECONST.DEFAULT_TOPICNAME;     // Addressが未定義の場合は、デフォルト値を使用
                        edge_FW._C_EgFW_PrintLog(EGAPI.EGFW.ServiceName, edge_FW.EG_LOG_WRN, string.Format("topic name is not set. default:[{0}]", MyTags.TopicName));
                    }
                    else
                    {
                        MyTags.TopicName = (string)value;                   // 取得値を使用
                    }
                }
                else
                {
                    // 生成されていない場合はデフォルト値でタグ生成する
                    MyTags.TopicName = EDGECONST.DEFAULT_TOPICNAME;         // デフォルト値
                    result = EGAPI.EgTagCreateEx(EGCONST.PROP_MYSERV_TOPICNAME, (ushort)EGDEFINE.egTagDataType.Boolean, MyTags.TopicName, "");
                    result = EGAPI.EgTagWrite(EGCONST.PROP_MYSERV_TOPICNAME, MyTags.TopicName);
                    edge_FW._C_EgFW_PrintLog(EGAPI.EGFW.ServiceName, edge_FW.EG_LOG_WRN, string.Format("{0} tag not exist. Tag was generated by default. ", EGCONST.PROP_MYSERV_TOPICNAME));
                }

                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービス開始
        /// </summary>
        /// <returns>正常時０</returns>
        public int StartService()
        {
            try
            {
                MyTags.Run = true;      // サービス実行
                UpdateIndicator();      // サービスインジケータ更新
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービスの初期化
        /// </summary>
        /// <returns>正常時０</returns>
        public Int32 InitService()
        {
            try
            {
                Int32 result = 0;
                LOG("[LOG] Start " + EGCONST.SERVICENAME + " Service");
                edge_API.EDGECONFIG config = new edge_API.EDGECONFIG(); //設定情報の初期化
                config = EGAPI.EDGE_CONFIG_DEFAULT;                     //初期値設定

                config.mInstMessFunc = MsgHandler;                      //Edgeメッセージユーザハンドラの指定

                result = EGAPI.EgInit(config);                          // Edgeフレームワークの初期化
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                {
                    WARN(result);
                    return result;
                }

                EGCONST.SET_EDGECONST(EGAPI.EGFW.ServiceRealName);

                result = GetProperty();                                 // サービスプロパティのロード処理
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                result = CreateServiceTags();                           // サービス提供タグの生成
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                MyTags.State = EDGECONST.SERVICE_UP;                    // サービスアップ
                StartService();

                // ライブインジケータタグの周期更新スタート / Service Live counter timer
                ServiceIndLiveTimer = new Timer(1000);
                ServiceIndLiveTimer.Elapsed += OnTimedLiveIndicaterEvent;
                ServiceIndLiveTimer.Start();

                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービス削除（プロセス停止）
        /// </summary>
        /// <returns></returns>
        public int KillService()
        {
            try
            {
                MyTags.Run = false;
                MyTags.State = EDGECONST.SERVICE_DOWN;
                UpdateIndicator();          // サービスインジケータ更新

                EGAPI.EgFinalize();         // APIのFinalize処理

                ServiceIndLiveTimer.Stop(); // Liveカウンタ用タイマー停止

                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        /// <summary>
        /// 自身のメールボックスにメッセージを送信する
        /// </summary>
        /// <param name="inMsgNo">メッセージNo</param>
        /// <returns></returns>
        public int SendMyMbox(int inMsgNo)
        {
            int inRet = 0;

            inRet = EGAPI.EgMailboxSend(EGAPI.EGFW.ServiceMboxName, EGAPI.EGFW.ServiceMboxName, inMsgNo);
            edge_FW._C_EgFW_PrintLog(EGAPI.EGFW.ServiceRealName, edge_FW.EG_LOG_DBG, "EgMailboxSend " + inMsgNo.ToString());

            return inRet;
        }


        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        /// <param name="filePath"></param>
        /// <param name="lineNumber"></param>
        public void WARN(int reason,
                         [CallerFilePath] string filePath = "",
                         [CallerLineNumber] int lineNumber = 0)
        {
            try
            {
                Console.WriteLine(EGCONST.SERVICENAME + " service warning at file(" + filePath + ") ,line(" + lineNumber.ToString() +
                                   ")  ,reason 0x" + reason.ToString("x8"));
            }
            catch
            { }
        }
        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        public void LOG(string message)
        {
            try
            {
                Console.WriteLine(EGCONST.SERVICENAME + " : " + message);
            }
            catch
            { }
        }

    }
}
